@@ -22,6 +22,11 @@
 #include "polarssl/x509.h"
 #include "polarssl/version.h"
 
+#define SSL_HANDSHAKE_OKE       0
+#define SSL_HANDSHAKE_ERROR    -1
+#define SSL_HANDSHAKE_TIMEOUT  -2
+#define SSL_HANDSHAKE_NO_MATCH -3
+
 #if POLARSSL_VERSION_NUMBER < 0x01030000
 #define x509_crt x509_cert
 #define pk_context rsa_context
@@ -57,6 +62,7 @@ int  ssl_send(ssl_context *ssl, const char *buffer, unsigned int length);
 bool ssl_has_peer_cert(ssl_context *context);
 int  get_peer_cert_info(ssl_context *context, char *subject_dn, char *issuer_dn, char *serial_nr, int length);
 char *ssl_version_string(ssl_context *context);
+char *ssl_cipher_string(ssl_context *context);
 void ssl_close(ssl_context *ssl);
 void ssl_shutdown(void);
 #ifdef ENABLE_RPROXY
